@@ -4,21 +4,60 @@ Exception definitions.
 """
 
 
+class UnsupportedVersion(Exception):
+    """Indicates that the user is trying to use an unsupported
+    version of the API"""
+    pass
+
+
 class CommandError(Exception):
     pass
 
 
+class AuthorizationFailure(Exception):
+    pass
+
+
+class NoUniqueMatch(Exception):
+    pass
+
+
+class NoTokenLookupException(Exception):
+    """This form of authentication does not support looking up
+       endpoints from an existing token."""
+    pass
+
+
+class EndpointNotFound(Exception):
+    """Could not find Service or Region in Service Catalog."""
+    pass
+
+
+class AmbiguousEndpoints(Exception):
+    """Found more than one matching endpoint in Service Catalog."""
+    def __init__(self, endpoints=None):
+        self.endpoints = endpoints
+
+    def __str__(self):
+        return "AmbiguousEndpoints: %s" % repr(self.endpoints)
+
+
 class ClientException(Exception):
     """
     The base exception class for all exceptions this library raises.
     """
-    def __init__(self, code, message=None, details=None):
+    def __init__(self, code, message=None, details=None, request_id=None):
         self.code = code
         self.message = message or self.__class__.message
         self.details = details
+        self.request_id = request_id
 
     def __str__(self):
-        return "%s (HTTP %s)" % (self.message, self.code)
+        formatted_string = "%s (HTTP %s)" % (self.message, self.code)
+        if self.request_id:
+            formatted_string += " (Request-ID: %s)" % self.request_id
+
+        return formatted_string
 
 
 class BadRequest(ClientException):
@@ -93,6 +132,7 @@ def from_response(response, body):
             raise exception_from_response(resp, body)
     """
     cls = _code_map.get(response.status, ClientException)
+    request_id = response.get('x-compute-request-id')
     if body:
         message = "n/a"
         details = "n/a"
@@ -100,6 +140,7 @@ def from_response(response, body):
             error = body[body.keys()[0]]
             message = error.get('message', None)
             details = error.get('details', None)
-        return cls(code=response.status, message=message, details=details)
+        return cls(code=response.status, message=message, details=details,
+                   request_id=request_id)
     else:
-        return cls(code=response.status)
+        return cls(code=response.status, request_id=request_id)
